"""identifier for data load

Revision ID: 954def4c1b92
Revises: 601c0b812f50
Create Date: 2018-08-22 13:56:11.419775

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '954def4c1b92'
down_revision = '601c0b812f50'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('data', sa.Column('load_name', sa.Text))


def downgrade():
    op.drop_column('data', 'load_name')



"""views-start

Revision ID: 53fb4e66bf09
Revises: a6a6cdd6ba67
Create Date: 2018-10-05 16:15:48.495745

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = '53fb4e66bf09'
down_revision = 'a6a6cdd6ba67'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###

    create_view_statements = '''
    create view "offer"
     as
    select 
      offer.value->>'id' as offer_id,
      collection,
      id as deal_table_id,
      deal_id,
      deal->>'title' deal_title,
      deal->>'status' deal_status,
      offer.value as offer 
    from 
      deal, 
      jsonb_array_elements(deal->'offers') offer;
     
    create view "project"
     as
    select 
      project.value->>'id' as project_id,
      collection,
      id as deal_table_id,
      deal_id,
      deal->>'title' deal_title,
      deal->>'status' deal_status,
      project.value as project 
    from 
      deal, 
      jsonb_array_elements(deal->'projects') project;

    create view "grant"
     as
    select 
      "grant".value->>'id' as grant_id,
      deal->'investments'->> 'summary' as investment_summary,
      collection,
      id as deal_table_id,
      deal_id,
      deal->>'title' deal_title,
      deal->>'status' deal_status,
      "grant".value as grant 
    from 
      deal, 
      jsonb_array_elements(deal->'investments'->'grants') "grant";

    create view "equity"
     as
    select 
      equity.value->>'id' as equity_id,
      deal->'investments'->> 'summary' as investment_summary,
      collection,
      id as deal_table_id,
      deal_id,
      deal->>'title' deal_title,
      deal->>'status' deal_status,
      equity.value as equity 
    from 
      deal, 
      jsonb_array_elements(deal->'investments'->'equity') equity;

    create view "credit"
     as
    select 
      "credit".value->>'id' as credit_id,
      deal->'investments'->> 'summary' as investment_summary,
      collection,
      id as deal_table_id,
      deal_id,
      deal->>'title' deal_title,
      deal->>'status' deal_status,
      "credit".value as credit 
    from 
      deal, 
      jsonb_array_elements(deal->'investments'->'credit') credit;
    '''

    op.execute(create_view_statements)

    # ### end Alembic commands ###

def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    drop_view_statements = '''
    drop view "offer";
    drop view "project";
    drop view "grant";
    drop view "equity";
    drop view "credit";
    '''

    op.execute(drop_view_statements)
    # ### end Alembic commands ###



"""Prep for actual data

Revision ID: 601c0b812f50
Revises: 09789f11b2fd
Create Date: 2018-08-20 19:25:35.836266

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects.postgresql import JSONB


# revision identifiers, used by Alembic.
revision = '601c0b812f50'
down_revision = '09789f11b2fd'
branch_labels = None
depends_on = None


def upgrade():
    op.create_table('data',
                    sa.Column('id', sa.Integer, primary_key=True),
                    sa.Column('date_loaded', sa.DateTime),
                    sa.Column('data', JSONB, nullable=False),
                    )
    op.drop_table('meta')


def downgrade():
    op.create_table('meta',
                    sa.Column('load_datetime', sa.DateTime),
                    sa.Column('data_source', sa.Text)
                    )
    op.drop_table('data')



"""aggregate_views

Revision ID: 6c67a8bea936
Revises: 53fb4e66bf09
Create Date: 2018-10-08 14:38:30.383613

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = '6c67a8bea936'
down_revision = '53fb4e66bf09'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    create_view_statements = '''
    create view offer_summary
    as
    select 
      deal_table_id, 
      count(*) offer_count,
      sum(coalesce((offer.offer->>'investmentTarget')::numeric, 0)) investment_target,
      sum(coalesce((offer.offer->>'minimumInvestmentTarget')::numeric, 0)) minimum_investment_target,
      sum(coalesce((offer.offer->>'maximumInvestmentTarget')::numeric, 0)) maximum_investment_target
    from offer group by 1;


    create view project_summary
    as
    select 
        project.deal_table_id,
        count(*) as project_count,
        sum(asset_count) asset_count,
        sum(asset_value) asset_value
    from project 
    left join 
       (select 
          deal_table_id, 
          count(*) as asset_count, 
          sum(coalesce((asset.value->>'totalValue')::numeric, 0)) asset_value 
       from 
          project, 
          jsonb_array_elements(project->'assets') as asset group by 1
       ) assets 
       on assets.deal_table_id = project.deal_table_id 
    group by 1;


    create view grant_summary
    as
    select 
      deal_table_id, 
      count(*) grant_count,
      sum(coalesce(("grant"."grant"->>'amountCommitted')::numeric, 0)) grant_amount_committed
    from "grant" group by 1;


    create view equity_summary
    as
    select 
      deal_table_id, 
      count(*) equity_count,
      sum(coalesce(("equity"."equity"->>'value')::numeric, 0)) equity_value
    from "equity" group by 1;


    create view credit_summary
    as
    select 
      deal_table_id, 
      count(*) credit_count,
      sum(coalesce(("credit"."credit"->>'estimatedValue')::numeric, 0)) credit_estimated_value
    from "credit" group by 1;


    create view deal_summary
    as
    select 
       id deal_table_id,
       deal_id,
       collection,
       coalesce(offer_count, 0) as offer_count,
       coalesce(investment_target, 0) as investment_target,
       coalesce(minimum_investment_target, 0) as minimum_investment_target,
       coalesce(maximum_investment_target, 0) as maximum_investment_target,
       coalesce(project_count, 0) as project_count,
       coalesce(asset_count, 0) as asset_count,
       coalesce(asset_value, 0) as asset_value,
       coalesce(grant_count, 0) as grant_count,
       coalesce(grant_amount_committed, 0) as grant_amount_committed,
       coalesce(equity_count, 0) as equity_count,
       coalesce(equity_value, 0) as equity_value,
       coalesce(credit_count, 0) as credit_count,
       coalesce(credit_estimated_value, 0) as credit_estimated_value
    from deal
       left join offer_summary os on os.deal_table_id = deal.id
       left join project_summary ps on ps.deal_table_id = deal.id
       left join grant_summary gs on gs.deal_table_id = deal.id
       left join equity_summary es on es.deal_table_id = deal.id
       left join credit_summary cs on cs.deal_table_id = deal.id;


    create view collection_summary
    as
    select 
       collection,
       count(*) deal_count,
       sum(offer_count) offer_count,
       sum(investment_target) investment_target,
       sum(minimum_investment_target) minimum_investment_target,
       sum(maximum_investment_target) maximum_investment_target,
       sum(project_count) project_count,
       sum(asset_count) asset_count,
       sum(asset_value) asset_value,
       sum(grant_count) grant_count,
       sum(grant_amount_committed) grant_amount_committed,
       sum(equity_count) equity_count,
       sum(equity_value) equity_value,
       sum(credit_count) credit_count,
       sum(credit_estimated_value) credit_estimated_value
    from deal_summary
    group by 1
    '''

    op.execute(create_view_statements)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    drop_view_statements = '''
    drop view collection_summary;
    drop view deal_summary;
    drop view credit_summary;
    drop view equity_summary;
    drop view grant_summary;
    drop view project_summary;
    drop view offer_summary;
    '''

    op.execute(drop_view_statements)
    # ### end Alembic commands ###



import datetime
import html
import json
import os

import alembic.config
import jinja2
from flattentool import unflatten

from sedldata.database import db


def in_notebook():
    if 'JPY_PARENT_PID' in os.environ:
        return True
    return False


def xl_to_json(infile, outfile):
    try:
        unflatten(
                input_name=infile,
                output_name=outfile,
                input_format='xlsx',
                metatab_name='Meta',
                metatab_vertical_orientation=True,
                root_list_path='deals',
                id_name='id',
                cell_source_map='sourcemap-' + outfile,
                root_id='')
        with open(outfile,'r') as json_file:
            data = json.load(json_file)
        with open('sourcemap-' + outfile,'r') as source_map:
            source_map_data = json.load(source_map)
    except Exception as e:
        raise e

    return data, source_map_data


def generate_migration(name):
    print("Generating database migrations")

    alembic_cfg_path = os.path.abspath(os.path.join(
        os.path.dirname(__file__), 'alembic.ini'))
    alembicargs = [
        '--config', alembic_cfg_path,
        '--raiseerr',
        'revision', '--autogenerate', '-m', name
    ]
    alembic.config.main(argv=alembicargs)


def load(infile, outfile, name):
    if name is None:
        name = infile
    # Load something into the database
    now = datetime.datetime.now()
    unflattened = xl_to_json(infile, outfile)
    insert = db.insert()
    insert.execute(date_loaded=now, load_name=name, data=unflattened)
    print("Loaded %s at: %s" % (name, now))


def load_xlsx(collection=None, infile=None, outfile='output.json'):
    if not collection and in_notebook():
        collection = input('Please state collections name: ')
    if not collection:
        raise ValueError('You need to input a non-empty collection name!')

    if in_notebook():
        from google.colab import files
        print('Upload your xlsx SEDL file:')
        uploaded = files.upload()
        for file_name in uploaded:
            infile = 'uploaded.xlsx'
            with open(infile, '+wb') as f:
                f.write(uploaded[file_name])
            break
    
    if not infile:
        raise ValueError('You need to state an input file')

    unflattened, source_map = xl_to_json(infile, outfile)
    deal_indexes = set()
    org_indexes = set()

    for path, value in source_map.items():
        split_path = path.split('/')
        if len(split_path) < 2:
            continue
        index = int(split_path[1])
        for source_item in value:
            if source_item[0].lower().strip().startswith('deal'):
                deal_indexes.add(index)
            if source_item[0].lower().strip().startswith('org'):
                org_indexes.add(index)

    metadata = {key: value for key, value in unflattened.items() if key != 'deals'}
    for num, obj in enumerate(unflattened['deals']):
        now = datetime.datetime.now()
        obj_id = obj.get('id')
        if not obj_id:
            print(num)
            print('WARNING: object {} has no id field'.format(obj))
            continue
        if num in deal_indexes:
            obj_id = obj.get('id')
            insert = db.deal_table.insert()
            insert.execute(date_loaded=now, collection=collection, deal=obj, deal_id=obj_id, metadata=metadata)
        if num in org_indexes:
            obj_id = obj['id']
            insert = db.org_table.insert()
            insert.execute(date_loaded=now, collection=collection, organization=obj, org_id=obj_id, metadata=metadata)

    now = datetime.datetime.now()
    print("Loaded %s at: %s" % (collection, now))


def delete_collection(collection):
    run_sql('''delete from deal where collection = %s ''', params=[collection])
    run_sql('''delete from organization where collection = %s ''', params=[collection])


table = jinja2.Template(
'''
<table class="dataframe">
    <thead>
    <tr>
      {% for header in headers %}
        <th style="text-align: left; vertical-align: top">{{ header }}</th>
      {% endfor %}
    </tr>
    </thead>
    <tbody>
      {% for row in data %}
        <tr>
          {% for cell in row %}
              <td style="text-align: left; vertical-align: top">
                <pre>{{ cell }}</pre>
              </td>
          {% endfor %}
        </tr>
      {% endfor %}
    </tbody>
</table>
'''
)


def generate_rows(result, limit):
    for num, row in enumerate(result):
        if num == limit:
            break
        yield [json.dumps(item, indent=2) if isinstance(item, dict) else html.escape(str(item)) for item in row]


def get_results(sql, limit=-1, params=None):

    with db.engine.begin() as connection:
        params = params or []
        sql_result = connection.execute(sql, *params)
        if sql_result.returns_rows:
            results = {
                "data": [row for row in generate_rows(sql_result, limit)],
                "headers": sql_result.keys()
            }
            return results
        else:
            return "Success"


def run_sql(sql, limit=100, params=None):
    from IPython.core.display import display, HTML
    results = get_results(sql, limit, params)
    if results == 'Success':
        return results
    display(HTML(table.render(results)))



"""deal-and-org

Revision ID: a6a6cdd6ba67
Revises: 954def4c1b92
Create Date: 2018-10-04 22:40:23.085634

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = 'a6a6cdd6ba67'
down_revision = '954def4c1b92'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('deal',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('collection', sa.Text(), nullable=False),
    sa.Column('deal_id', sa.Text(), nullable=False),
    sa.Column('date_loaded', sa.DateTime(), nullable=True),
    sa.Column('deal', postgresql.JSONB(astext_type=sa.Text()), nullable=False),
    sa.Column('metadata', postgresql.JSONB(astext_type=sa.Text()), nullable=True),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('organization',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('collection', sa.Text(), nullable=False),
    sa.Column('org_id', sa.Text(), nullable=False),
    sa.Column('date_loaded', sa.DateTime(), nullable=True),
    sa.Column('organization', postgresql.JSONB(astext_type=sa.Text()), nullable=False),
    sa.Column('metadata', postgresql.JSONB(astext_type=sa.Text()), nullable=True),
    sa.PrimaryKeyConstraint('id')
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('orgainization')
    op.drop_table('deal')
    # ### end Alembic commands ###



"""Init

Revision ID: 09789f11b2fd
Revises: 
Create Date: 2018-08-13 15:13:07.081787

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '09789f11b2fd'
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    op.create_table('meta',
    				sa.Column('load_datetime', sa.DateTime),
    				sa.Column('data_source', sa.Text)
    	)


def downgrade():
    op.drop_table('meta')



import getpass

from sedldata.database import db
from sedldata.lib import delete_collection
from sedldata.lib import load, load_xlsx, in_notebook, run_sql, get_results, generate_migration


if in_notebook():
    db_uri = 'postgresql://sedldata:{}@46.43.2.250:5432/sedldata'.format(getpass.getpass("Enter database password:  "))
    db.init_db(db_uri)
    db.upgrade()
else:
    db.init_db()





from setuptools import setup

setup(
    name='sedldata',
    version='0.0.2',
    license='BSD',
    packages=['sedldata', 'sedldata.migrate', 'sedldata.migrate.versions'],
    package_data={'sedldata': ['alembic.ini'], 'sedldata.migrate': 'script.py.mako'},
    install_requires=[
        'Click',
        'SQLAlchemy',
        'alembic',
        'psycopg2',
        'configparser',
        'jinja2',
        'requests',
        'flattentool'
    ],
    entry_points='''
        [console_scripts]
        sedldata=sedldata.cli:cli
    ''',
)



import os
from configparser import ConfigParser

import alembic.config
import sqlalchemy as sa
from sqlalchemy.dialects.postgresql import JSONB


class Database:

    def __init__(self):
        self.engine = None
        self.metadata = None
        self.deal_table = None
        self.org_table = None

    def config(self, filename='database.ini', section='postgresql'):
        parser = ConfigParser()
        parser.read(filename)

        db = {}
        if parser.has_section(section):
            params = parser.items(section)
            for param in params:
                db[param[0]] = param[1]
        else:
            raise Exception(
                'Section {0} not found in the {1} file'.format(section, filename))

        return db

    def create_db_uri(self):
        db_uri = os.environ.get('DB_URI')
        if db_uri is None:
            db_params = self.config()
            db_uri = 'postgresql://{0}:{1}@{2}:{3}/{4}'.format(
                db_params['user'],
                db_params['password'],
                db_params['host'],
                db_params['port'],
                db_params['database'])

        return db_uri

    def init_db(self, db_uri=None):
        if self.engine:
            return

        if not db_uri:
            db_uri = self.create_db_uri()
        self.engine = sa.create_engine(db_uri)
        self.metadata = sa.MetaData(bind=self.engine)

        self.deal_table = sa.Table(
            'deal', self.metadata,
            sa.Column('id', sa.Integer, primary_key=True),
            sa.Column('collection', sa.Text, nullable=False),
            sa.Column('deal_id', sa.Text, nullable=False),
            sa.Column('date_loaded', sa.DateTime),
            sa.Column('deal', JSONB, nullable=False),
            sa.Column('metadata', JSONB)
        )

        self.org_table = sa.Table(
            'organization', self.metadata,
            sa.Column('id', sa.Integer, primary_key=True),
            sa.Column('collection', sa.Text, nullable=False),
            sa.Column('org_id', sa.Text, nullable=False),
            sa.Column('date_loaded', sa.DateTime),
            sa.Column('organization', JSONB, nullable=False),
            sa.Column('metadata', JSONB)
        )

    def upgrade(self):
        # Let alembic create the tables
        print("Upgrading database")

        alembic_cfg_path = os.path.abspath(os.path.join(
            os.path.dirname(__file__), 'alembic.ini'))
        alembicargs = [
            '--config', alembic_cfg_path,
            '--raiseerr',
            'upgrade', 'head',
        ]
        alembic.config.main(argv=alembicargs)


db = Database()



import os
import click
import datetime

import alembic.config

from sedldata.database import datatable
from sedldata.lib import upgrade as lib_upgrade
from sedldata.lib import load as lib_load


@click.group()
def cli():
    pass


@cli.command()
def upgrade():
    lib_upgrade()


@cli.command()
@click.argument('infile')
@click.argument('outfile')
@click.option('--name', default=None)
def load(infile, outfile, name):
    lib_load(infile, outfile, name)


@cli.command()
def dump():
    # Dump the datatable
    click.echo("Dump\n")

    select = datatable.select()
    rows = select.execute()
    for row in rows:
        print(row)
